# workerプロセス数はCPU数の3倍とする
cpu_num = `cat /proc/cpuinfo | grep processor`.split("\n").size
worker_processes 3 * cpu_num
# ログファイル
stderr_path File.expand_path('log/unicorn-err.log', ENV['RAILS_ROOT'])
stdout_path File.expand_path('log/unicorn.log', ENV['RAILS_ROOT'])
pid File.expand_path('tmp/pids/unicorn.pid', ENV['RAILS_ROOT'])
# リクエスト処理に10秒以上かかったworkerプロセスはmasterにkillされ再起動される
timeout 10

# masterプロセスでアプリケーションを読み込んでおくことでメモリ使用量を減らすことができます。
# ただし以下のbefore_fork, after_forkでのリソース開放、初期化処理が必要になります。
preload_app true

before_fork do |server, _worker|
  # masterプロセスではリクエストを処理しないため、DBやRedisへの無駄なコネクションは切っておく
  ActiveRecord::Base.connection.disconnect! if defined?(ActiveRecord::Base)
  Qiita::Redis.quit
  # ダウンタイムのないデプロイ用の設定。デプロイ時はmasterプロセスにUSR2シグナルを送る。
  # masterプロセスがUSR2シグナルを受け取ると、新規にmasterプロセスが立ち上がり、
  # 古いmasterプロセスのpidファイルはファイル名に.oldbinを加えて保存するため、
  # `if old_pid != server.pid`で新しいmasterプロセスが立ち上がっているのか
  # チェックする。立ち上がっていれば古いmasterプロセスにQUITシグナルを送り、
  # graceful shutdown（処理中のリクエストが完了したらプロセスを終了）する。
  # シグナルについて：http://unicorn.bogomips.org/SIGNALS.html
  old_pid = "#{server.config[:pid]}.oldbin"
  if old_pid != server.pid
    begin
      Process.kill :QUIT, File.read(old_pid).to_i
    rescue Errno::ENOENT, Errno::ESRCH
      nil
    end
  end
end

# Forkしてできたworkerプロセスで行う処理
after_fork do |_server, worker|
  # workerプロセスで改めてDBやRedisとの接続を確立する。
  ActiveRecord::Base.establish_connection if defined?(ActiveRecord::Base)
  load Rails.root.join('config/initializers/redis.rb')
end
